################################################
#
# PDF_ERRORS
#
################################################

from math import sqrt

# Calculate pdf-error with 90% prescription from ct10
def pdf_error_ct10(data):
	sum_nlop = 0.0
	sum_nlom = 0.0

	for i in range(1, int((len(data)+1.0)/2.0)):
		sum_nlop += max((data[2*i - 1][1] - data[0][1]), (data[2*i][1]-data[0][1]), 0)**(2.0)
		sum_nlom += max((data[0][1] - data[2*i - 1][1]), (data[0][1] - data[2*i][1]), 0)**(2.0)

	C90 = 1.64485

	return (data[0][1], sqrt(sum_nlop)/C90, sqrt(sum_nlom)/C90)

# Calculate pdf-error for mstw and lhce 
def pdf_error_mstw(data):
	sum_nlop = 0.0
	sum_nlom = 0.0

	for i in range(1, int((len(data))/2.0)):
		sum_nlop += max((data[2*i - 1][1] - data[0][1]), (data[2*i][1]-data[0][1]), 0)**(2.0)
		sum_nlom += max((data[0][1] - data[2*i - 1][1]), (data[0][1] - data[2*i][1]), 0)**(2.0)

	return (data[0][1], sqrt(sum_nlop), sqrt(sum_nlom))

# Calculate pdf-error as mean and deviation over all
# available pdf-members, except the 0'th one.
# Method for nnpdf
def pdf_error_nnpdf(data):
	central = data[0]

	sum_nlo = 0.0
	sum_nlo2 = 0.0

	for d in data:
		# Don't take that!
		if d == central:
			continue

		sum_nlo += d[1]
		sum_nlo2 += d[1]**2.0

	N = float(len(data)-1)

	sum_nlo = sum_nlo/N
	sum_nlo2 = sum_nlo2/N

	return (
	         sum_nlo, sqrt(N * (sum_nlo2 - sum_nlo**2.0) / ( N - 1)), sqrt(N * (sum_nlo2 - sum_nlo**2.0) / ( N - 1)) )

# Symmetric error prescription for abkm
def pdf_error_abkm(data):
	central = data[0]

	s = 0.0

	for d in data:
		if d == central:
			continue

		s += (d[1]-central[1])**2.0

	return (central[1], sqrt(s), sqrt(s))



################################################
#
# TABLE CREATION 
#
################################################

import re

def search_line(stri, f):
	line = ""
	res = None
	while True:
		line = f.readline()
		res = re.match(stri, line)
		if res:
			break
		if len(line) == 0:
			print "End of file reached while searching \"%s\"" % stri
			#exit()
			raise RuntimeError("Result not found.")

	return line, res

import os.path as path

def create_tables():
	pdfsets = {}

	for i in range(1, 15318):
		if not path.isfile("jobs/job_propaganda%d" % i):
			print "File job_propaganda%d missing." % d
			continue

		print "Processing job_propaganda%d" % i
		f = open("jobs/job_propaganda%d" % i, "r")

		try:
			search_line("\s*and for nlo:", f)
		except:
			continue

		line, res = search_line("\s*([^.]+)\.", f)
		pdf_name = res.group(1)
#		if pdf_name == "cteq6":
#			pdf_name = "CTEQ6"

		if pdf_name == "NNPDF21_100":
			pdf_name = "NNPDF21"
		elif pdf_name == "MSTW2008nlo68cl":
			pdf_name = "MSTW2008"
		elif pdf_name == "LHECNLO_EIG":
			pdf_name = "LHECNLO"
		elif pdf_name == "HERAPDF10_EIG":
			pdf_name = "HERAPDF10"
		elif pdf_name == "abkm09_5_nlo":
			pdf_name = "ABKM09"
		elif pdf_name == "HERAPDF15NLO_EIG":
			pdf_name = "HERAPDF15"
		elif pdf_name == "LHECNLO_EIG32":
			pdf_name = "LHEC"

		line, res = search_line("\s*set\s*(\d+)", f)
		pdf_set = int(res.group(1))

		if not pdf_name in pdfsets.viewkeys():
			pdfsets[pdf_name] = {}

		pdf_data = pdfsets[pdf_name]

		line, res = search_line("\s*M_g\s*=\s*([0-9.]+)", f)
		M_g = float(res.group(1))

		if not M_g in pdf_data.viewkeys():
			pdf_data[M_g] = []

		data = pdf_data[M_g]

		try:
			search_line("\s*lo\s*rel_error\s*nlo\s*rel_error\s*K\s*nlo\(susy\)\s*",f)
			line, res = search_line("\s*(?P<LO>([0-9.Ee+]|[-])+)\s*(?P<LOerr>([0-9.Ee+]|[-])+)\s*(?P<NLO>([0-9.Ee+]|[-])+)\s*(?P<NLOerr>([0-9.Ee+]|[-])+)\s*(?P<K>([0-9.Ee+]|[-])+)\s*(?P<NLOsusy>([0-9.Ee+]|[-])+)", f)
		except:
			continue

		data.append((pdf_set, float(res.group("NLO")), float(res.group("NLOerr"))))
#		data.append((pdf_set, float(res.group("LO")), float(res.group("LOerr"))))

		f.close()
		
	for pdf_name in pdfsets:
		pdf_data = pdfsets[pdf_name]
		for mass_name in pdf_data:
			data = pdf_data[mass_name]
#			mass.sort(cmp = lambda x,y: x[0] < y[0])
			data = sorted(data, key = lambda x: x[0])

			f = open("tables/%s_%dGeV" % (pdf_name, mass_name), "w")
		
			f.write("# member    NLO           relative_err  \n")

			for d in data:
				f.write("%3d %10e %10e\n" % d)

			f.close()

################################################
#
#  Reading of tables
#
################################################

def read_table(name):
	f = open("tables/"+name, "r")

	# drop first line
	for line in f:
		break

	data = []

	for line in f:
		l = line.split()
		data.append((int(l[0]), float(l[1])))

	return data


import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties as font

##################################################################
#
# Actual plotting 
#
##################################################################


pdfs = [("CT10", 53, pdf_error_ct10),
        ("LHECNLO", 35, pdf_error_mstw),
        ("MSTW2008", 41, pdf_error_mstw),
        ("NNPDF21", 101, pdf_error_nnpdf),
        ("HERAPDF10", 21, pdf_error_mstw),
        ("LHEC", 33, pdf_error_mstw),
        ("HERAPDF15", 21, pdf_error_mstw),
        ("ABKM09", 26, pdf_error_abkm)
	  ]


def get_pdf_error(pdf, m_g):
	if type(pdf) == type(""):
		for p in pdfs:
			if p[0] == pdf:
				return get_pdf_error(p, m_g)

	data = read_table("%s_%dGeV" % (pdf[0], m_g))
	res = pdf[2](data)

	return res

def plot_error_tube(plot_x, plot_y, color, legend = None, no_central = False):
	# This checks the value of the error, for
	# LHEC central-value - lower sigma goes under
	# 0.0. Thats not good.
	for i in plot_y:
		if i[0] < i[2]:
			print i[0], i[1]
			print plot_x[plot_y.index(i)]
			raise RuntimeError("Central-value is smaller then lower error... Really bad...")

	if not no_central:
		plt.plot(plot_x, [i[0] for i in plot_y], color, linestyle="-", linewidth=2.0, label=legend)
		plt.plot(plot_x, [i[0] +  i[1] for i in plot_y], color, linestyle="--", linewidth=2.0)
		plt.plot(plot_x, [i[0] - i[2] for i in plot_y], color, linestyle="--", linewidth=2.0)
	else:
		plt.plot(plot_x, [i[0] +  i[1] for i in plot_y], color, linestyle="--", linewidth=2.0, label=legend)
		plt.plot(plot_x, [i[0] - i[2] for i in plot_y], color, linestyle="--", linewidth=2.0)


def analysis():
	plot_x = []
	plot_cteq = []
	plot_lhec = []
	plot_mstw = []
	plot_nnpdf = []
	plot_hera = []
	plot_abkm = []

	for i in range(0, 46):
		m_g = 500 + i * (4500.0/45.0)
		plot_x.append(m_g)
		plot_cteq.append(get_pdf_error("CT10", m_g))		
#		plot_lhec.append(get_pdf_error("LHECNLO", m_g))		
		plot_lhec.append(get_pdf_error("LHEC", m_g))		
		plot_mstw.append(get_pdf_error("MSTW2008", m_g))		
		plot_nnpdf.append(get_pdf_error("NNPDF21", m_g))		
#		plot_hera.append(get_pdf_error("HERAPDF10", m_g))		
		plot_hera.append(get_pdf_error("HERAPDF15", m_g))		
		plot_abkm.append(get_pdf_error("ABKM09", m_g))		


	# This turns absolute values in mstw-ratios. 
	for i in range(0,len(plot_x)):
		plot_cteq[i] = (plot_cteq[i][0]/plot_mstw[i][0], plot_cteq[i][1]/plot_mstw[i][0], plot_cteq[i][2]/plot_mstw[i][0])
		plot_lhec[i] = (plot_lhec[i][0]/plot_mstw[i][0], plot_lhec[i][1]/plot_lhec[i][0], plot_lhec[i][2]/plot_lhec[i][0])
		plot_nnpdf[i] = (plot_nnpdf[i][0]/plot_mstw[i][0], plot_nnpdf[i][1]/plot_nnpdf[i][0], plot_nnpdf[i][2]/plot_nnpdf[i][0])
		plot_hera[i] = (plot_hera[i][0]/plot_mstw[i][0], plot_hera[i][1]/plot_hera[i][0], plot_hera[i][2]/plot_hera[i][0])
		plot_abkm[i] = (plot_abkm[i][0]/plot_mstw[i][0], plot_abkm[i][1]/plot_abkm[i][0], plot_abkm[i][2]/plot_abkm[i][0])
		plot_mstw[i] = (plot_mstw[i][0]/plot_mstw[i][0], plot_mstw[i][1]/plot_mstw[i][0], plot_mstw[i][2]/plot_mstw[i][0])

	# Plot central values with errors
#	plot_error_tube(plot_x, plot_cteq, "red", "CT10")
#	plot_error_tube(plot_x, plot_lhec, "green", "LHEC", no_central = True)
#	plot_error_tube(plot_x, plot_mstw, "black", "MSTW2008")
#	plot_error_tube(plot_x, plot_nnpdf, "cyan", "NNPDF21")
#	plot_error_tube(plot_x, plot_hera, "magenta", "HERA15")
#	plot_error_tube(plot_x, plot_abkm, "blue", "ABKM09")


	# Plot central values without errors
	plt.plot(plot_x, [i[0] for i in plot_cteq], color="red", label="CT10")
	plt.plot(plot_x, [i[0] for i in plot_hera], color="magenta", label="HERA15")
	plt.plot(plot_x, [i[0] for i in plot_mstw], color="black", label="MSTW2008")
	plt.plot(plot_x, [i[0] for i in plot_abkm], color="blue", label="ABKM09")
	plt.plot(plot_x, [i[0] for i in plot_nnpdf], color="cyan", label="NNPDF21")


	plt.legend(loc="best", prop=font(size=30))
	plt.xlim(500, 3000)
	plt.ylim(-1.0, 3.0)
	plt.xlabel("$m_g$ = $m_{sq}$ [GeV]", fontdict={"size":30})
	plt.ylabel(r"$\sigma$/$\sigma_{mstw}$",fontdict={"size":30})
	plt.title("PDF-Errors for pp -> gluino + gluino + X",fontdict={"size":30})
#	plt.semilogy()
	plt.show()

import sys

if __name__ == "__main__":
	if len(sys.argv) > 1:
		if sys.argv[1] == "tables":
			create_tables()
			exit()
		if sys.argv[1] == "ct_mstw":
			ct_err = get_pdf_error("CT10", 2500)
			print "CT10: %5e +%5e" % (ct_err[0], ct_err[1])
			print "                   -%5e" % ct_err[2]
			mstw_err = get_pdf_error("MSTW2008", 2500)
			print "MSTW: %5e +%5e" % (mstw_err[0], mstw_err[1])
			print "                   -%5e" % mstw_err[2]
		else:
			print "Unknown param: %s" % sys.argv[1]
			exit()

	analysis()


